@@ -80,7 +80,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, signals => {
     ),
   ];
 
-our $VERSION = '1.8.5';
+our $VERSION = '1.8.6';
 
 my $SANE_NAME_PAGE_HEIGHT = SANE_NAME_PAGE_HEIGHT;
 my $SANE_NAME_PAGE_WIDTH  = SANE_NAME_PAGE_WIDTH;
@@ -387,6 +387,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
                 if ($val) { $widget->set_active(TRUE) }
                 $widget->{signal} = $widget->signal_connect(
                     toggled => sub {
+                        $self->{num_reloads} = 0;    # num-reloads is read-only
                         my $value = $widget->get_active;
                         $self->set_option( $opt, $value );
                     }
@@ -398,6 +399,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
                 $widget = Gtk2::Button->new( $d_sane->get( $opt->{title} ) );
                 $widget->{signal} = $widget->signal_connect(
                     clicked => sub {
+                        $self->{num_reloads} = 0;    # num-reloads is read-only
                         $self->set_option( $opt, $val );
                     }
                 );
@@ -419,6 +421,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
                 }
                 $widget->{signal} = $widget->signal_connect(
                     'value-changed' => sub {
+                        $self->{num_reloads} = 0;    # num-reloads is read-only
                         my $value = $widget->get_value;
                         $self->set_option( $opt, $value );
                     }
@@ -443,6 +446,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
                 if ( defined $index ) { $widget->set_active($index) }
                 $widget->{signal} = $widget->signal_connect(
                     changed => sub {
+                        $self->{num_reloads} = 0;    # num-reloads is read-only
                         my $i = $widget->get_active;
                         $self->set_option( $opt, $opt->{constraint}[$i] );
                     }
@@ -459,13 +463,14 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
                 }
                 $widget->{signal} = $widget->signal_connect(
                     activate => sub {
+                        $self->{num_reloads} = 0;    # num-reloads is read-only
                         my $value = $widget->get_text;
                         $self->set_option( $opt, $value );
                     }
                 );
             }
         }
-        else {    # $opt->{max_values} > 1
+        else {                                       # $opt->{max_values} > 1
             $widget = Gtk2::Button->new( $d_sane->get( $opt->{title} ) );
             $widget->{signal} = $widget->signal_connect(
                 clicked => \&multiple_values_button_callback,
@@ -678,9 +683,7 @@ sub set_option {
                         }
                         $self->signal_emit( 'changed-options-cache', $cache );
                     }
-                    if ($options) {
-                        $self->update_options( $options, $option->{name} );
-                    }
+                    if ($options) { $self->update_options($options) }
 
                     $self->signal_emit( 'finished-process',
                         'find_scan_options' );
